# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: sparkplug.proto
# Protobuf Python Version: 5.27.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC, 5, 27, 3, "", "sparkplug.proto"
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0fsparkplug.proto\x12!com.cirruslink.sparkplug.protobuf"\xf6\x16\n\x07Payload\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x42\n\x07metrics\x18\x02 \x03(\x0b\x32\x31.com.cirruslink.sparkplug.protobuf.Payload.Metric\x12\x0b\n\x03seq\x18\x03 \x01(\x04\x12\x0c\n\x04uuid\x18\x04 \x01(\t\x12\x0c\n\x04\x62ody\x18\x05 \x01(\x0c\x1a\xbe\x04\n\x08Template\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x42\n\x07metrics\x18\x02 \x03(\x0b\x32\x31.com.cirruslink.sparkplug.protobuf.Payload.Metric\x12Q\n\nparameters\x18\x03 \x03(\x0b\x32=.com.cirruslink.sparkplug.protobuf.Payload.Template.Parameter\x12\x14\n\x0ctemplate_ref\x18\x04 \x01(\t\x12\x15\n\ris_definition\x18\x05 \x01(\x08\x1a\xd2\x02\n\tParameter\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\r\x12\x13\n\tint_value\x18\x03 \x01(\rH\x00\x12\x14\n\nlong_value\x18\x04 \x01(\x04H\x00\x12\x15\n\x0b\x66loat_value\x18\x05 \x01(\x02H\x00\x12\x16\n\x0c\x64ouble_value\x18\x06 \x01(\x01H\x00\x12\x17\n\rboolean_value\x18\x07 \x01(\x08H\x00\x12\x16\n\x0cstring_value\x18\x08 \x01(\tH\x00\x12p\n\x0f\x65xtension_value\x18\t \x01(\x0b\x32U.com.cirruslink.sparkplug.protobuf.Payload.Template.Parameter.ParameterValueExtensionH\x00\x1a#\n\x17ParameterValueExtension*\x08\x08\x01\x10\x80\x80\x80\x80\x02\x42\x07\n\x05value*\x08\x08\x06\x10\x80\x80\x80\x80\x02\x1a\xaf\x04\n\x07\x44\x61taSet\x12\x16\n\x0enum_of_columns\x18\x01 \x01(\x04\x12\x0f\n\x07\x63olumns\x18\x02 \x03(\t\x12\r\n\x05types\x18\x03 \x03(\r\x12\x44\n\x04rows\x18\x04 \x03(\x0b\x32\x36.com.cirruslink.sparkplug.protobuf.Payload.DataSet.Row\x1a\xb7\x02\n\x0c\x44\x61taSetValue\x12\x13\n\tint_value\x18\x01 \x01(\rH\x00\x12\x14\n\nlong_value\x18\x02 \x01(\x04H\x00\x12\x15\n\x0b\x66loat_value\x18\x03 \x01(\x02H\x00\x12\x16\n\x0c\x64ouble_value\x18\x04 \x01(\x01H\x00\x12\x17\n\rboolean_value\x18\x05 \x01(\x08H\x00\x12\x16\n\x0cstring_value\x18\x06 \x01(\tH\x00\x12p\n\x0f\x65xtension_value\x18\x07 \x01(\x0b\x32U.com.cirruslink.sparkplug.protobuf.Payload.DataSet.DataSetValue.DataSetValueExtensionH\x00\x1a!\n\x15\x44\x61taSetValueExtension*\x08\x08\x01\x10\x80\x80\x80\x80\x02\x42\x07\n\x05value\x1a\x62\n\x03Row\x12Q\n\x08\x65lements\x18\x01 \x03(\x0b\x32?.com.cirruslink.sparkplug.protobuf.Payload.DataSet.DataSetValue*\x08\x08\x02\x10\x80\x80\x80\x80\x02*\x08\x08\x05\x10\x80\x80\x80\x80\x02\x1a\x81\x04\n\rPropertyValue\x12\x0c\n\x04type\x18\x01 \x01(\r\x12\x0f\n\x07is_null\x18\x02 \x01(\x08\x12\x13\n\tint_value\x18\x03 \x01(\rH\x00\x12\x14\n\nlong_value\x18\x04 \x01(\x04H\x00\x12\x15\n\x0b\x66loat_value\x18\x05 \x01(\x02H\x00\x12\x16\n\x0c\x64ouble_value\x18\x06 \x01(\x01H\x00\x12\x17\n\rboolean_value\x18\x07 \x01(\x08H\x00\x12\x16\n\x0cstring_value\x18\x08 \x01(\tH\x00\x12S\n\x11propertyset_value\x18\t \x01(\x0b\x32\x36.com.cirruslink.sparkplug.protobuf.Payload.PropertySetH\x00\x12X\n\x12propertysets_value\x18\n \x01(\x0b\x32:.com.cirruslink.sparkplug.protobuf.Payload.PropertySetListH\x00\x12j\n\x0f\x65xtension_value\x18\x0b \x01(\x0b\x32O.com.cirruslink.sparkplug.protobuf.Payload.PropertyValue.PropertyValueExtensionH\x00\x1a"\n\x16PropertyValueExtension*\x08\x08\x01\x10\x80\x80\x80\x80\x02\x42\x07\n\x05value\x1ao\n\x0bPropertySet\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12H\n\x06values\x18\x02 \x03(\x0b\x32\x38.com.cirruslink.sparkplug.protobuf.Payload.PropertyValue*\x08\x08\x03\x10\x80\x80\x80\x80\x02\x1ah\n\x0fPropertySetList\x12K\n\x0bpropertyset\x18\x01 \x03(\x0b\x32\x36.com.cirruslink.sparkplug.protobuf.Payload.PropertySet*\x08\x08\x02\x10\x80\x80\x80\x80\x02\x1a\xa4\x01\n\x08MetaData\x12\x15\n\ris_multi_part\x18\x01 \x01(\x08\x12\x14\n\x0c\x63ontent_type\x18\x02 \x01(\t\x12\x0c\n\x04size\x18\x03 \x01(\x04\x12\x0b\n\x03seq\x18\x04 \x01(\x04\x12\x11\n\tfile_name\x18\x05 \x01(\t\x12\x11\n\tfile_type\x18\x06 \x01(\t\x12\x0b\n\x03md5\x18\x07 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x08 \x01(\t*\x08\x08\t\x10\x80\x80\x80\x80\x02\x1a\xe7\x05\n\x06Metric\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05\x61lias\x18\x02 \x01(\x04\x12\x11\n\ttimestamp\x18\x03 \x01(\x04\x12\x10\n\x08\x64\x61tatype\x18\x04 \x01(\r\x12\x15\n\ris_historical\x18\x05 \x01(\x08\x12\x14\n\x0cis_transient\x18\x06 \x01(\x08\x12\x0f\n\x07is_null\x18\x07 \x01(\x08\x12\x45\n\x08metadata\x18\x08 \x01(\x0b\x32\x33.com.cirruslink.sparkplug.protobuf.Payload.MetaData\x12J\n\nproperties\x18\t \x01(\x0b\x32\x36.com.cirruslink.sparkplug.protobuf.Payload.PropertySet\x12\x13\n\tint_value\x18\n \x01(\rH\x00\x12\x14\n\nlong_value\x18\x0b \x01(\x04H\x00\x12\x15\n\x0b\x66loat_value\x18\x0c \x01(\x02H\x00\x12\x16\n\x0c\x64ouble_value\x18\r \x01(\x01H\x00\x12\x17\n\rboolean_value\x18\x0e \x01(\x08H\x00\x12\x16\n\x0cstring_value\x18\x0f \x01(\tH\x00\x12\x15\n\x0b\x62ytes_value\x18\x10 \x01(\x0cH\x00\x12K\n\rdataset_value\x18\x11 \x01(\x0b\x32\x32.com.cirruslink.sparkplug.protobuf.Payload.DataSetH\x00\x12M\n\x0etemplate_value\x18\x12 \x01(\x0b\x32\x33.com.cirruslink.sparkplug.protobuf.Payload.TemplateH\x00\x12\x61\n\x0f\x65xtension_value\x18\x13 \x01(\x0b\x32\x46.com.cirruslink.sparkplug.protobuf.Payload.Metric.MetricValueExtensionH\x00\x1a \n\x14MetricValueExtension*\x08\x08\x01\x10\x80\x80\x80\x80\x02\x42\x07\n\x05value*\x08\x08\x06\x10\x80\x80\x80\x80\x02\x42\x34\n!com.cirruslink.sparkplug.protobufB\x0fSparkplugBProto'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "sparkplug_pb2", _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    _globals["DESCRIPTOR"]._loaded_options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n!com.cirruslink.sparkplug.protobufB\017SparkplugBProto"
    _globals["_PAYLOAD"]._serialized_start = 55
    _globals["_PAYLOAD"]._serialized_end = 2989
    _globals["_PAYLOAD_TEMPLATE"]._serialized_start = 195
    _globals["_PAYLOAD_TEMPLATE"]._serialized_end = 769
    _globals["_PAYLOAD_TEMPLATE_PARAMETER"]._serialized_start = 421
    _globals["_PAYLOAD_TEMPLATE_PARAMETER"]._serialized_end = 759
    _globals[
        "_PAYLOAD_TEMPLATE_PARAMETER_PARAMETERVALUEEXTENSION"
    ]._serialized_start = 715
    _globals[
        "_PAYLOAD_TEMPLATE_PARAMETER_PARAMETERVALUEEXTENSION"
    ]._serialized_end = 750
    _globals["_PAYLOAD_DATASET"]._serialized_start = 772
    _globals["_PAYLOAD_DATASET"]._serialized_end = 1331
    _globals["_PAYLOAD_DATASET_DATASETVALUE"]._serialized_start = 910
    _globals["_PAYLOAD_DATASET_DATASETVALUE"]._serialized_end = 1221
    _globals[
        "_PAYLOAD_DATASET_DATASETVALUE_DATASETVALUEEXTENSION"
    ]._serialized_start = 1179
    _globals[
        "_PAYLOAD_DATASET_DATASETVALUE_DATASETVALUEEXTENSION"
    ]._serialized_end = 1212
    _globals["_PAYLOAD_DATASET_ROW"]._serialized_start = 1223
    _globals["_PAYLOAD_DATASET_ROW"]._serialized_end = 1321
    _globals["_PAYLOAD_PROPERTYVALUE"]._serialized_start = 1334
    _globals["_PAYLOAD_PROPERTYVALUE"]._serialized_end = 1847
    _globals["_PAYLOAD_PROPERTYVALUE_PROPERTYVALUEEXTENSION"]._serialized_start = 1804
    _globals["_PAYLOAD_PROPERTYVALUE_PROPERTYVALUEEXTENSION"]._serialized_end = 1838
    _globals["_PAYLOAD_PROPERTYSET"]._serialized_start = 1849
    _globals["_PAYLOAD_PROPERTYSET"]._serialized_end = 1960
    _globals["_PAYLOAD_PROPERTYSETLIST"]._serialized_start = 1962
    _globals["_PAYLOAD_PROPERTYSETLIST"]._serialized_end = 2066
    _globals["_PAYLOAD_METADATA"]._serialized_start = 2069
    _globals["_PAYLOAD_METADATA"]._serialized_end = 2233
    _globals["_PAYLOAD_METRIC"]._serialized_start = 2236
    _globals["_PAYLOAD_METRIC"]._serialized_end = 2979
    _globals["_PAYLOAD_METRIC_METRICVALUEEXTENSION"]._serialized_start = 2938
    _globals["_PAYLOAD_METRIC_METRICVALUEEXTENSION"]._serialized_end = 2970
# @@protoc_insertion_point(module_scope)
